function J = jacF(Y,data)
%
% Compute the Jacobian matrix in Y of the mapping F for the BVP 
%
% Parameters in :
% Y : point in which the Jcobian is evaluated (vector in R^4)
% data : table of the fiber laser parameter values
% Parameters out :
% J : Jacobian matrix of F in Y (4x4 matrix)
%
c_data=num2cell(data);
[nu_p,nu_s,Psat_p,Psat_s,alpha_a_p,alpha_a_s,alpha_p,alpha_s,P0,Psat_e_s]=deal(c_data{[2:9,17,18]});
%
den= 1+(Y(1)+Y(2))/Psat_p + (Y(3)+Y(4))/Psat_s;
Gp=(alpha_a_p*(Y(1)+Y(2))+(nu_p/nu_s)*alpha_a_s*(Y(3)+Y(4)))/den ;
Gs=(alpha_a_s*(Y(3)+Y(4))+(nu_s/nu_p)*alpha_a_p*(Y(1)+Y(2)))/den ;
dGp=grad_Gp(Y,data);
dGs=grad_Gs(Y,data);
%
J(1,1) = -(alpha_a_p+alpha_p) + (Gp/Psat_p) + (Y(1)/Psat_p)*dGp(1);
J(1,2:4) =  transpose( (Y(1)/Psat_p)*dGp(2:4) );
J(2,2) = (alpha_a_p+alpha_p) - (Gp/Psat_p) - (Y(2)/Psat_p)*dGp(2);
J(2,[1,3,4]) = - transpose( (Y(2)/Psat_p).*dGp([1,3,4]) );
J(3,3) =  -(alpha_a_s+alpha_s) + (Gs/Psat_s) + (Y(3)/Psat_s + P0/Psat_e_s)*dGs(3);
J(3,[1,2,4]) =  transpose( (Y(3)/Psat_s + P0/Psat_e_s).*dGs([1,2,4]) );
J(4,4) =  (alpha_a_s+alpha_s) - (Gs/Psat_s) -(Y(4)/Psat_s + P0/Psat_e_s)*dGs(4);
J(4,1:3) = - transpose( (Y(4)/Psat_s + P0/Psat_e_s).*dGs(1:3) );
end

%%
function dGp=grad_Gp(Y,data)
%
% Compute the gradient of the mapping G_p in Y
%
% Parameters in :
% Y : point in which the Jcobian is evaluated (vector in R^4)
% data : table of the fiber laser parameter values
% Parameters out :
% dGp : gradient vector of G_p in Y  (vector in R^4)
%
c_data=num2cell(data);
[nu_p,nu_s,Psat_p,Psat_s,alpha_a_p,alpha_a_s]=deal(c_data{[2:7]});
%
den= 1+(Y(1)+Y(2))/Psat_p + (Y(3)+Y(4))/Psat_s;
Gp=(alpha_a_p*(Y(1)+Y(2))+(nu_p/nu_s)*alpha_a_s*(Y(3)+Y(4)))/den ;
%
d1Gp=(alpha_a_p - Gp/Psat_p)/den;
d2Gp=d1Gp;
d3Gp=((nu_p/nu_s)*alpha_a_s - Gp/Psat_s)/den;
d4Gp=d3Gp;
dGp=[d1Gp;d2Gp;d3Gp;d4Gp];
end

%%
function dGs=grad_Gs(Y,data)
%
% Compute the gradient of the mapping G_s in Y
%
% Parameters in :
% Y : point in which the Jcobian is evaluated (vector in R^4)
% data : table of the fiber laser parameter values
% Parameters out :
% dGs : gradient vector of G_s in Y  (vector in R^4)
%
c_data=num2cell(data);
[nu_p,nu_s,Psat_p,Psat_s,alpha_a_p,alpha_a_s]=deal(c_data{[2:7]});
%
den= 1+(Y(1)+Y(2))/Psat_p + (Y(3)+Y(4))/Psat_s;
Gs=(alpha_a_s*(Y(3)+Y(4))+(nu_s/nu_p)*alpha_a_p*(Y(1)+Y(2)))/den ;
%
d1Gs=((nu_s/nu_p)*alpha_a_p - Gs/Psat_p)/den;
d2Gs=d1Gs;
d3Gs=(alpha_a_s - Gs/Psat_s)/den;
d4Gs=d3Gs;
dGs=[d1Gs;d2Gs;d3Gs;d4Gs];
end

