clear
close all
% Universal constants
hh = 6.62607015E-34; % Planck constant in unit J s
cc= 299792458; % speed of light in vacuum in unit m/s
% -------------------------------------------------------------------------
% Fiber parameter values
% -------------------------------------------------------------------------
data_file = input('[?] Name of the fiber parameters data file : ','s');
run(data_file);
% intermediary constants defined from the fiber data
nu_p=cc/lambda_p; 
nu_s=cc/lambda_s; 
Psat_p= Aeff*hh*nu_p/( (sigma_a_p+sigma_e_p)*tau3*Gamma_p);
Psat_s= Aeff*hh*nu_s/( (sigma_a_s+sigma_e_s)*tau3*Gamma_s);
Psat_e_s=Aeff*hh*nu_s/( sigma_e_s*tau3*Gamma_s);
alpha_a_p=sigma_a_p*Gamma_p*Nt;
alpha_a_s=sigma_a_s*Gamma_s*Nt;
%
% Table of data used in the various functions
data=[L,nu_p,nu_s,Psat_p,Psat_s,alpha_a_p,alpha_a_s,alpha_p,alpha_s,0,...
      R1_s,R2_s,Ppump_p,Ppump_m,R1_p,R2_p,P0,Psat_e_s];
t0=cputime;
% -------------------------------------------------------------------------
% Computation of the guess for BVP solver
% -------------------------------------------------------------------------
% Computation of the initial condition for the equivalent IVP when losses
% and spontaneous emission are neglected
A=(nu_p/nu_s)*(Psat_s/Psat_p)*(0.5*log(R1_s*R2_s)-alpha_a_s*L)+alpha_a_p*L;
Y02=(Ppump_p*R2_p+Ppump_m*exp(A))/(exp(2*A)-R1_p*R2_p);
eta=sqrt(R1_s/R2_s)-R1_s-sqrt(R1_s*R2_s)+1;
if R2_p==0
    PHI=(exp(-A)-1)*(Ppump_p+Ppump_m);
else
    phi=(-0.5*Ppump_m+sqrt(0.25*Ppump_m^2+ R1_p*R2_p*Y02^2+Ppump_p*R2_p*Y02))/R2_p;
    PHI=phi*(1-R2_p)-(Ppump_p+Ppump_m)+Y02*(1-R1_p);
end
Y04=(Psat_s/eta)*(0.5*log(R1_s*R2_s)-alpha_a_s*L-(nu_s/nu_p)*PHI/Psat_s);
Y0_ivp = [R1_p*Y02+Ppump_p;Y02;R1_s*Y04;Y04]; % initial condition for the IVP on Y  
options = odeset('RelTol',1e-9,'AbsTol',1e-12); % options for ODE45
data([8,9,17])=0; % Losses are set to zero as well as spontaneous emission to solve the IVP
[z_ivp,Y1] = ode45(@(z,y) bvpfcn(y,data),[0,L],Y0_ivp,options); % solves the IVP
data([8,9,17])=[alpha_p,alpha_s,P0]; % Losses and spontaneous emission are set back to their original values
% -------------------------------------------------------------------------
% Solving of the BVP
% -------------------------------------------------------------------------
solinit = bvpinit(z_ivp,@(z)guess(z,z_ivp,Y1)); % provides the guess for BVP4C
options = bvpset('RelTol',1e-9,'AbsTol',1e-12,'FJacobian', @(z,Y)jacF(Y,data)); % options for BVP4C
sol = bvp4c(@(z,Y) bvpfcn(Y,data), @(Y_0,Y_L) bcfcn(Y_0,Y_L,data), solinit,options); % solve the BVP
disp(' ')
disp(['Done! Overall simulation CPU time = ',num2str(cputime-t0),' s.']);
disp(' ');
% -------------------------------------------------------------------------
% Results display
% -------------------------------------------------------------------------
disp(['Forward pump power at z=0 :   P^+_p(0) = ',num2str(sol.y(1,1),6),' W']);
disp(['Backward pump power at z=0 :  P^-_p(0) = ',num2str(sol.y(2,1),6),' W']);
disp(['Forward laser power at z=0 :  P^+_s(0) = ',num2str(sol.y(3,1),6),' W']);
disp(['Backward laser power at z=0 : P^-_s(0) = ',num2str(sol.y(4,1),6),' W']);
disp(['Forward pump power at z=L :   P^+_p(L) = ',num2str(sol.y(1,end),6),' W']);
disp(['Backward pump power at z=L :  P^-_p(L) = ',num2str(sol.y(2,end),6),' W']);
disp(['Forward laser power at z=L :  P^+_s(L) = ',num2str(sol.y(3,end),6),' W']);
disp(['Backward laser power at z=L : P^-_s(L) = ',num2str(sol.y(4,end),6),' W']);
Ppumpabs=nu_s/nu_p*(sol.y(1,1)-sol.y(1,end)+sol.y(2,end)-sol.y(2,1));
Plaser = (1-R2_s)*sol.y(3,end)+(1-R1_s)*sol.y(4,1);
disp(' ');
disp(['Theoretical available power at ',num2str(lambda_p*1E9,4),' nm : ',num2str(Ppumpabs,6),' W']);
disp(['Total laser output power (at z=0 and z=L) : ',num2str(Plaser,6),' W']);
disp(' ');
% Error estimator
den =  abs(sol.y(1,1)*sol.y(2,1)+sol.y(1,end)*sol.y(2,end));
if abs(den)>eps
    err_p = abs(sol.y(1,1)*sol.y(2,1)-sol.y(1,end)*sol.y(2,end))/den;
    disp(['Error estimator E_p = ',num2str(err_p)])
end
if (P0==0)
    err_s = abs(sol.y(3,1)*sol.y(4,1)-sol.y(3,end)*sol.y(4,end)) ...
        / abs(sol.y(3,1)*sol.y(4,1)+sol.y(3,end)*sol.y(4,end));
    disp(['Error estimator E_s = ',num2str(err_s)])
end
%
% Graphical display
%
figure; hold on; grid
plot(sol.x,sol.y(1,:),'-b','Linewidth',2)
plot(sol.x,sol.y(2,:),'-c','Linewidth',2)
plot(sol.x,sol.y(3,:),'-r','Linewidth',2)
plot(sol.x,sol.y(4,:),'-m','Linewidth',2)
xlabel('Position along the fiber [m]')
ylabel('Power [W]')
N2=(tau3/Aeff)*((alpha_a_p/(hh*nu_p))*(sol.y(1,:)+sol.y(2,:))+(alpha_a_s/(hh*nu_s))*(sol.y(3,:)+sol.y(4,:)))./ ...
    (1+(sol.y(1,:)+sol.y(2,:))/Psat_p + (sol.y(3,:)+sol.y(4,:))/Psat_s);
N2=N2/Nt;
yyaxis right
plot(sol.x,N2,'-.','Linewidth',2);
legend('P_p^+','P_p^-','P_s^+','P_s^-','N_2/N');
ylabel('ratio N_2/N')

%%
% -------------------------------------------------------------------------
% Internal functions for BVP4C
% -------------------------------------------------------------------------
function dYdz = bvpfcn(Y,data) 
%
% Definition of system of first-order equations defining the BVP equation
% Provide the function F such that the BVP equation reads U'(z) = F(z,U)
%
c_data=num2cell(data);
[nu_p,nu_s,Psat_p,Psat_s,alpha_a_p,alpha_a_s,alpha_p,alpha_s,P0,Psat_e_s]=deal(c_data{[2:9,17,18]});
%
den= 1+(Y(1)+Y(2))/Psat_p + (Y(3)+Y(4))/Psat_s;
Gp=(alpha_a_p*(Y(1)+Y(2))+(nu_p/nu_s)*alpha_a_s*(Y(3)+Y(4)))/den;
Gs=(alpha_a_s*(Y(3)+Y(4))+(nu_s/nu_p)*alpha_a_p*(Y(1)+Y(2)))/den;
%
dY(1) = -(alpha_p+alpha_a_p)*Y(1) + Gp*(Y(1)/Psat_p);
dY(2) =  (alpha_p+alpha_a_p)*Y(2) - Gp*(Y(2)/Psat_p);
dY(3) = -(alpha_s+alpha_a_s)*Y(3) + Gs*(Y(3)/Psat_s + P0/Psat_e_s);
dY(4) =  (alpha_s+alpha_a_s)*Y(4) - Gs*(Y(4)/Psat_s + P0/Psat_e_s);
dYdz=transpose(dY);
end
%--------------------------------
function res = bcfcn(U_0,U_L,data) 
%
% Code Boundary Conditions for the equivalent modified BVP with separated
% boundary conditions 
%
% Parameters in :
% U_0 : solution value at fiber front end
% U_L : solution value at fiber back end 
% data : table of the fiber laser parameter values
% Parameters out :
% res : boundary condition function value for U_0 and U_L

c_data=num2cell(data);
[R1_s,R2_s,Ppump_p,Ppump_m,R1_p,R2_p]=deal(c_data{[11:16]});

res = [U_0(1)-R1_p*U_0(2)-Ppump_p
       U_L(2)-R2_p*U_L(1)-Ppump_m
       U_0(3)-R1_s*U_0(4)
       U_L(4)-R2_s*U_L(3)];
end
%--------------------------------
function g = guess(z,z_ivp,U)
%
% Returns interpolated values of the solution (z_ivp,U) to the IVP when
% losses and spontaneous emmission are neglected to create an initial
% guess for BVP4C on the grid defined by z
%
% Parameters in :
% z : contains the coordinates of the query points
% z_ivp : contains the sample points
% U : contains the corresponding values
% Parameters out :
% g : interpolated values at the query points

g = interp1(z_ivp,U,z);
end
